#!/usr/bin/env python3

from twp_vismach import *
import hal
import math
import sys

# Machine zero as measured from center surface of the rotary c table
machine_zero_x = -800
machine_zero_y = 0
machine_zero_z = 0


try: # Expect files in working directory
    # NOTE to run this file as standalone python script absolute paths might have to be used to find the stl files
    # create the work piece from file
    # work_piece = AsciiSTL(filename="./work_piece_1.stl")
    pass

except Exception as detail:
    print(detail)
    raise SystemExit("mazak-integrex-200y-gui requires stl files in working directory")

for setting in sys.argv[1:]: exec(setting)

c = hal.component("mazak-integrex-200y-gui")
# axis_x
c.newpin("axis_x", hal.HAL_FLOAT, hal.HAL_IN)
# axis_y
c.newpin("axis_y", hal.HAL_FLOAT, hal.HAL_IN)
# head vertical slide
c.newpin("axis_z", hal.HAL_FLOAT, hal.HAL_IN)
# rotary_b
c.newpin("rotary_b", hal.HAL_FLOAT, hal.HAL_IN)
# rotary_c
c.newpin("rotary_c", hal.HAL_FLOAT, hal.HAL_IN)
# tool offsets
c.newpin("tool_length", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("tool_diameter", hal.HAL_FLOAT, hal.HAL_IN)
# geometric offsets in the spindle-rotary-assembly
c.newpin("pivot_z", hal.HAL_FLOAT, hal.HAL_IN)
c.newpin("angle_y_yt", hal.HAL_FLOAT, hal.HAL_IN)
# selected kinematics
c.newpin("kinstype_select", hal.HAL_FLOAT, hal.HAL_IN)
# spindle body show/hide
c.newpin("hide_spindle_body", hal.HAL_BIT, hal.HAL_IN)
# scale coordinate system indicators
c.newpin("scale_coords", hal.HAL_FLOAT, hal.HAL_IN)
c.ready()


# give endpoint Z values and radii
# resulting cylinder is on the Z axis
class HalToolCylinder(CylinderZ):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 6 # default if hal pin not set
        if (c.tool_diameter > 0): r = c.tool_diameter/2
        return -c.tool_length, r, 0, r

# used to create the spindle housing as set by the variable 'pivot_z'
class HalSpindleHousingZ(CylinderZ):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 140
        length = c.pivot_z + 200
        # start the spindle housing at hight 100 above the spindle nose
        return length, r, 100, r

# used to create the spindle rotary as set by the variable 'pivot_y'
class HalSpindleHousingY(CylinderY):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 140
        length = 0
        return length, r, 0, r

# used to create an indicator for the variable 'pivot_z'
class HalPivotZ(CylinderZ):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 2
        length = c.pivot_z
        # start the spindle housing at hight 100 above the spindle nose
        return length, 1, 0, r


# used to create a thin tool-offset indicator
class HalToolOffset(CylinderZ):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 2
        return -c.tool_length, 1, 0, r

class CoordSystem(Collection):
    # creates a visual object for a coordinate system
    def __init__(self, comp, r=4, l=500):
        # set arrow length
        al = l/10
        self.parts = [Color([1,0,0,1],[CylinderX(0,r,l,r)]),
                      Color([1,0,0,1],[CylinderX(l,2*r,l+al,1)]),
                      Color([0,1,0,1],[CylinderY(0,r,l,r)]),
                      Color([0,1,0,1],[CylinderY(l,2*r,l+al,1)]),
                      Color([0,0,1,1],[CylinderZ(0,r,l,r)]),
                      Color([0,0,1,1],[CylinderZ(l,2*r,l+al,1)]),
                     ]

class Point(Collection):
    # creates a visual object for a point
    def __init__(self, comp, r=1, l=20, color=[1,1,1,1]):
        self.parts = [Color(color,[CylinderX(-l,r,l,r)],1),
                      Color(color,[CylinderY(-l,r,l,r)],1),
                      Color(color,[CylinderZ(-l,r,l,r)],1)
                     ]

#indicators
# create vismach coordinates to show the origin during construction of the model
vismach_coords = CoordSystem(c,2,5000)

# create absolute coordinates
abs_coords = CoordSystem(c,8,300)
abs_coords = HalScale([abs_coords],c,1,1,1,"scale_coords")
# create work coordinates that represent work_offset offset
work_coords = CoordSystem(c,2,100)
work_coords = HalScale([work_coords],c,1,1,1,"scale_coords")
work_coords= Rotate([work_coords],90,0,0,1)
# create a visual indicator for the position of the control point
ctrl_pt = Point(c)
ctrl_pt = HalTranslate([ctrl_pt],c,"tool_length",0,0,-1)
# indicators to be displayed when in identity mode
ident_coords = HalShow([work_coords],c,0,"kinstype_select")
# indicators to be displayed when in tcp mode
tcp_coords = HalShow([work_coords],c,1,"kinstype_select")
ident_coords = HalRotate([ident_coords],c,"angle_y_yt",1,0,0,1)
#/indicators

#tool-side
tooltip = Capture()
tool_cylinder = Collection([
                HalToolCylinder(c),
                # this indicates the spindle nose when no tool-offset is active
                CylinderZ(-0.1, 0, 0.1, 10),
                ])
tool_cylinder = HalShow([tool_cylinder],c,True,"hide_spindle_body",0,1)
# create an indicator for the tool-offset when spindle-body is hidden
ind_tool_offset = Color([1,0,1,0],[HalToolOffset(c)])
tool = Collection([
       HalTranslate([tooltip], c, "tool_length", 0,0,-1),
       tool_cylinder,
       ind_tool_offset
       ])
tool = Color([1,0,1,0], [tool] )
# create an indicator for the variable pivot_z
ind_pivot_z = Color([0,0,0.75,1],[HalPivotZ(CylinderZ)])
# create the spindle housing that contains the spindle
spindle_housing = Collection([
                  # spindle
                  Color([0.7,0.7,0,1],[CylinderZ(0,50,50,100)]),
                  Color([0.7,0.7,0,1],[CylinderZ(50,100,100,100)]),
                  # spindle housing vertical length as set by 'pivot_z'
                  Color([0.7,0.7,0,1],[HalSpindleHousingZ(CylinderZ)])
                  ])
spindle_housing = HalShow([spindle_housing],c,True,"hide_spindle_body",0,1)
spindle_housing = Collection([
                  tool,
                  spindle_housing,
                  ind_pivot_z
                  ])
# move the spindle and it's vertical housing by the values set for the pivot lengths
# so the vismach origin is in the pivot point
spindle_housing = HalVectorTranslate([spindle_housing],c,0,0,"pivot_z",-1)
spindle_housing_horizontal = Color([0.7,0.7,0,1],[HalSpindleHousingY(c)])
spindle_housing_horizontal = HalShow([spindle_housing_horizontal],c,True,"hide_spindle_body",0,1)

spindle_housing = Collection([
                  spindle_housing,
                  spindle_housing_horizontal
                  ])
# create a visual for the position of the spindle A,B pivot-point
ind_pivot_point = Collection([
                  Color([1,1,1,1],[CylinderX(-200,1,200,1)]),
                  Color([1,1,0.3,1],[CylinderY(0,2,3000,2)]),
                  Color([1,1,1,1],[CylinderZ(-200,1,200,1)]),
                  # arrow indicating the up position
                  Color([1,1,1,1],[CylinderZ(200,5,210,1)]),
                  # sphere for the actual pivot point
                  Color([1,1,0.3,1],[Sphere(0,0,0,5)])
                  ])
# create nutation joint  for the spindle housing
spindle_housing_nut = Collection([
                      Color([0.7,0.7,0,1],[CylinderY(0,145,-70,145)]),
                      Color([0.7,0.7,0,1],[CylinderY(-70,145,-150,60)])
                      ])
spindle_housing_nut = HalShow([spindle_housing_nut],c,True,"hide_spindle_body",0,1)
spindle_housing = Collection([
                  spindle_housing,
                  #spindle_housing_nut
                  ])

# create HAL-link for b-axis rotational joint"
spindle_housing = HalRotate([spindle_housing],c,"rotary_b",-1,0,1,0)
# create middle part b rotary and a nutating axis
spindle_rotary = Collection([
                 # cylinder for b axis
                 CylinderY(0,120,100,120),
                 CylinderY(100,120,300,160),
                 ])
spindle_rotary = Color([1,0.5,0,0], [spindle_rotary] )
spindle_rotary = HalShow([spindle_rotary],c,True,"hide_spindle_body",0,1)

# join the two parts to a spindle assembly
spindle_assembly = Collection([
                   spindle_rotary,
                   ind_pivot_point,
                   spindle_housing,
                   spindle_rotary,
                   ])

# rotate the tool-spindle so it is pointing in the negative z-direction
spindle_assembly= Rotate([spindle_assembly],90,0,1,0)
# add a block for the y-axis to the spindle_assembly
slide_x = Color([0.6,0.8,0.3,0], [Box(-250, 200, -250, 250, 300, 500)])
slide_x = HalShow([slide_x],c,True,"hide_spindle_body",0,1)
spindle_x = Collection([
             spindle_assembly,
             slide_x
             ])
# move the spindle and it's vertical housing by the values set for the pivot lengths
# so the vismach origin is in the center of the spindle nose
spindle_x = HalTranslate([spindle_x],c,"pivot_z",1,0,0)
# spindle head and x-slide move with x
spindle_x = HalTranslate([spindle_x],c,"axis_x",0,0,1)
# rotate the x-slide and tool-spindle by the slide-angle
spindle_x = Rotate([spindle_x],90,-1,0,0)
spindle_x = HalRotate([spindle_x],c,"angle_y_yt",1,1,0,0)
# spindle head and y-slide move with y
spindle_xy = HalTranslate([spindle_x],c,"axis_y",0,-1,0)
# spindle head and y-slide move with z
spindle_xyz = HalTranslate([spindle_xy],c,"axis_z",1,0,0)
# move the spindle_yz to it's designated home position
spindle_xyz = Translate([spindle_xyz], machine_zero_x, machine_zero_y, machine_zero_z)
#/tool-side

#work-side
work = Capture()

# create rotary_table_c
rotary_table_c = Color([0.1,0.7,0.9,0],[CylinderZ(0,920,-800,920)])
# create 'slots' in x and y on the rotary table
slot_x_w = Color([1,1,1,1],[BoxCenteredXY(1840, 10, 0.9)])
slot_y_w = Color([1,1,1,1],[BoxCenteredXY(10, 1840, 0.9)])
# create indicator of at positive end of x slot
slot_pocket = Color([1,1,1,1],[CylinderZ(0,20,1,20)])
slot_pocket = Translate([slot_pocket],900,0,0)
rotary_table_c = Collection([
                 work,
                 rotary_table_c,
                 slot_x_w,
                 slot_y_w,
                 slot_pocket,
                 tcp_coords
                 ])
rotary_table_c = HalRotate([rotary_table_c],c,"angle_y_yt",1,0,0,1)
# create HAL-link for c-axis rotational joint"
rotary_table_c = HalRotate([rotary_table_c],c,"rotary_c",-1,0,0,1)


table = Collection([
        rotary_table_c,
        ident_coords,
        # block that carries the rotary c
        Color([0.5,0.5,0.5,0], [Box(-1500, -1000, -1300, 1500, 1000, -800)])
        ])
# rotate so it alignes with the z-axis
table = Rotate([table],90,0,1,0)
table = Translate([table],-800,0,0)
#/work-side

#base
base = Collection([
       # base
       Box(-3500, -1000, -1500, 3500, 3000, -1300)
       ])
base = Color([0.2,0.2,0.2,1], [base] )
#/base

model = Collection([table, spindle_xyz, base,])



main(model, tooltip, work, size=4000, lat=-60, lon=25)
